@@ -12,7 +12,7 @@ use strict;
 use warnings;
 
 package Net::SIP::Dispatcher;
-use fields (
+use fields ( 
 	# interface to outside
 	'receiver',       # callback into upper layer
 	'legs',           # \@list of Net::SIP::Legs managed by dispatcher
@@ -21,20 +21,19 @@ use fields (
 	'domain2proxy',   # optional mapping between SIP domains and proxies (otherwise use DNS)
 	# internals
 	'do_retransmits', # flag if retransmits will be done (false for stateless proxy)
+	'outgoing_leg',   # Leg for outgoing_proxy
 	'queue',          # \@list of outstanding Net::SIP::Dispatcher::Packet
 	'response_cache', # Cache of responses, used to reply to retransmits
-	'disp_expire',    # expire/retransmit timer
 );
 
 use Net::SIP::Leg;
-use Net::SIP::Util ':all';
-use Errno qw(EHOSTUNREACH ETIMEDOUT ENOPROTOOPT EINVAL);
+use Net::SIP::Util qw( invoke_callback );
+use Errno qw(EHOSTUNREACH ETIMEDOUT);
 use IO::Socket;
 use List::Util 'first';
 use Net::DNS;
 use Carp 'croak';
 use Net::SIP::Debug;
-use Scalar::Util 'weaken';
 
 
 ###########################################################################
@@ -56,8 +55,7 @@ use Scalar::Util 'weaken';
 ###########################################################################
 sub new {
 	my ($class,$legs,$eventloop,%args) = @_;
-
-	my ($outgoing_proxy,$do_retransmits,$domain2proxy)
+	my ($outgoing_proxy,$do_retransmits,$domain2proxy) 
 		= delete @args{qw( outgoing_proxy do_retransmits domain2proxy )};
 	die "bad args: ".join( ' ',keys %args ) if %args;
 
@@ -68,11 +66,8 @@ sub new {
 	$domain2proxy ||= {};
 	foreach ( values %$domain2proxy ) {
 		if ( ref($_) ) { # should be \@list of [ prio,proto,ip,port ]
-		} elsif ( m{^(?:(udp|tcp):)?([^:]+)(?::(\d+))?$} ) {
-			my @proto = $1 ? ( $1 ) : ( 'udp','tcp' );
-			my $host = $2;
-			my $port = $3 || 5060;
-			$_ = [ map { [ -1, $_, $host, $port ] } @proto ];
+		} elsif ( m{^([\d\.]+)(?::(\d+))} ) {
+			$_ = [ [ -1,'udp',$1,$2 ||5060 ] ]
 		} else {
 			croak( "invalid entry in domain2proxy: $_" );
 		}
@@ -83,6 +78,7 @@ sub new {
 		legs => [],
 		queue  => [],
 		outgoing_proxy => undef,
+		outgoing_leg => undef,
 		response_cache => {},
 		do_retransmits => defined( $do_retransmits ) ? $do_retransmits : 1,
 		eventloop => $eventloop,
@@ -95,21 +91,16 @@ sub new {
 		my $leg = $self->_find_leg4addr( $outgoing_proxy )
 			|| die "cannot find leg for destination $outgoing_proxy";
 		$self->{outgoing_proxy} = $outgoing_proxy;
+		$self->{outgoing_leg}   = $leg;
 	}
 
 
 	# regularly prune queue
 	my $sub = sub {
-		my ($self,$timer) = @_;
-		if ( $self ) {
-			$self->queue_expire( $self->{eventloop}->looptime );
-		} else {
-			$timer->cancel;
-		}
+		my ($self,$loop) = @_;
+		$self->queue_expire($loop->looptime);
 	};
-	my $cb = [ $sub,$self ];
-	weaken( $cb->[1] );
-	$self->{disp_expire} = $self->add_timer( 1,$cb,1,'disp_expire' );
+	$self->add_timer( 1,[ $sub,$self,$eventloop ],1 );
 
 	return $self;
 }
@@ -118,86 +109,70 @@ sub new {
 # set receiver, e.g the upper layer which gets the incoming packets
 # received by the dispatcher
 # Args: ($self,$receiver)
-#   $receiver: object which has receive( Net::SIP::Leg,Net::SIP::Packet )
+#   $receiver: object which has receive( Net::SIP::Leg,Net::SIP::Packet ) 
 #     method to handle incoming SIP packets or callback
-#     might be undef - in this case the existing receiver will be removed
 # Returns: NONE
 ###########################################################################
 sub set_receiver {
 	my Net::SIP::Dispatcher $self = shift;
-	if ( my $receiver = shift ) {
-		if ( my $sub = UNIVERSAL::can($receiver,'receive' )) {
-			# Object with method receive()
-			$receiver = [ $sub,$receiver ]
-		}
-		$self->{receiver} = $receiver;
-	} else {
-		# remove receiver
-		$self->{receiver} = undef
+	my $receiver = shift;
+	if ( my $sub = UNIVERSAL::can($receiver,'receive' )) {
+		# Object with method receive()
+		$receiver = [ $sub,$receiver ]
 	}
-
+	$self->{receiver} = $receiver;
 }
 
 ###########################################################################
 # adds a leg to the dispatcher
 # Args: ($self,@legs)
-#  @legs: can be sockets, \%args for constructing or already
+#  @legs: can be sockets, \%args for constructing or already 
 #    objects of class Net::SIP::Leg
 # Returns: NONE
 ###########################################################################
 sub add_leg {
 	my Net::SIP::Dispatcher $self = shift;
 	my $legs = $self->{legs};
-	foreach my $data (@_) {
+	foreach my $leg (@_) {
+
+		# copy before modifying
+		my $leg = $leg; 
 
-		my $leg;
 		# if it is not a leg yet create one based
 		# on the arguments
-		if ( UNIVERSAL::isa( $data,'Net::SIP::Leg' )) {
+		if ( UNIVERSAL::isa( $leg,'Net::SIP::Leg' )) {
 			# already a leg
-			$leg = $data;
 
 		} elsif ( UNIVERSAL::isa( $leg,'IO::Handle' )) {
 			# create from socket
-			$leg = Net::SIP::Leg->new( sock => $data );
+			my $sock = $leg;
+			$leg = Net::SIP::Leg->new( sock => $sock );
 
 		} elsif ( UNIVERSAL::isa( $leg,'HASH' )) {
 			# create from %args
-			$leg = Net::SIP::Leg->new( %$data );
-		} else {
-			croak "invalid spec for leg: $data";
+			my %args = %$leg;
+			$leg = Net::SIP::Leg->new( %args );
 		}
 
 		push @$legs, $leg;
 
-		if ( my $fd = $leg->fd ) {
-			my $cb = sub {
-				# don't crash Dispatcher on bad or unexpected packets
-				eval {
-					my ($self,$leg) = @_;
-					$self || return;
+		my $cb = sub {
+			my ($self,$leg) = @_;
 
-					# leg->receive might return undef if the packet wasnt
-					# read successfully. for tcp connections the receive
-					# on a listening socket might cause a new leg to be added
-					# which then will receive the packet (maybe over multiple
-					# read attempts)
-					my ($packet,$from) = $leg->receive or do {
-						DEBUG( 50,"failed to receive on leg" );
-						return;
-					};
-
-					# handle received packet
-					$self->receive( $packet,$leg,$from );
-				};
-				if ($@) {
-					DEBUG(1,"dispatcher croaked: $@");
-				}
+			# leg->receive might return undef if the packet wasnt
+			# read successfully. for tcp connections the receive
+			# on a listening socket might cause a new leg to be added
+			# which then will receive the packet (maybe over multiple
+			# read attempts)
+			my ($packet,$from) = $leg->receive or do {
+				DEBUG( "failed to receive" );
+				return;
 			};
-			$cb = [ $cb,$self,$leg ];
-			weaken( $cb->[1] );
-			$self->{eventloop}->addFD( $fd, $cb );
-		}
+
+			# handle received packet
+			$self->receive( $packet,$leg,$from );
+		};
+		$self->{eventloop}->addFD( $leg->{sock}, [ $cb,$self,$leg ]);
 	}
 }
 
@@ -212,9 +187,7 @@ sub remove_leg {
 	my $legs = $self->{legs};
 	foreach my $leg (@_) {
 		@$legs = grep { $_ != $leg } @$legs;
-		if ( my $fd = $leg->fd ) {
-			$self->{eventloop}->delFD( $fd );
-		}
+		$self->{eventloop}->delFD( $leg->{sock} );
 	}
 }
 
@@ -234,8 +207,6 @@ sub remove_leg {
 ###########################################################################
 sub get_legs {
 	my Net::SIP::Dispatcher $self = shift;
-	return @{ $self->{legs} } if ! @_; # shortcut
-
 	my %args = @_;
 	my @rv;
 	foreach my $leg (@{ $self->{legs} }) {
@@ -248,12 +219,12 @@ sub get_legs {
 	}
 	return @rv;
 }
-
+	
 
 ###########################################################################
 # add timer
-# propagates to add_timer of eventloop
-# Args: ($self,$when,$cb,$repeat)
+# propagates to addTimer of eventloop
+# Args: ($self,$when,$cb,$repeat) 
 #   $when: when callback gets called, can be absolute time (epoch, time_t)
 #     or relative time (seconds)
 #   $cb: callback
@@ -263,9 +234,9 @@ sub get_legs {
 ###########################################################################
 sub add_timer {
 	my Net::SIP::Dispatcher $self = shift;
-	return $self->{eventloop}->add_timer( @_ );
+	return $self->{eventloop}->addTimer( @_ );
 }
-
+	
 ###########################################################################
 # initiate delivery of a packet, e.g. put packet into delivery queue
 # Args: ($self,$packet,%more_args)
@@ -275,7 +246,7 @@ sub add_timer {
 #     callback:  [ \&sub,@arg ] for calling back on definite delivery
 #       success (tcp only) or error (timeout,no route,...)
 #     leg:       specify outgoing leg, needed for responses
-#     dst_addr:  specify outgoing addr [ip,port] or sockaddr, needed
+#     dst_addr:  specify outgoing addr [ip,port] or sockaddr, needed 
 #       for responses
 #     do_retransmits: if retransmits should be done, default from
 #        global value (see new())
@@ -289,20 +260,16 @@ sub deliver {
 	my $do_retransmits = delete $more_args{do_retransmits};
 	$do_retransmits = $self->{do_retransmits} if !defined $do_retransmits;
 
-	DEBUG( 100,"deliver $packet" );
+	DEBUG( "deliver $packet" );
 
 	if ( $packet->is_response ) {
 		# cache response for 32 sec (64*T1)
-		if ( $do_retransmits ) {
-			my $cid = join( "\0",
-				map { $packet->get_header($_) }
-				qw( cseq call-id from to )
-			);
-			$self->{response_cache}{$cid} = {
-				packet => $packet,
-				expire => ( $now ||= time()) +32
-			};
-		}
+		my $cid = $packet->get_header( 'cseq' )
+			."\0".$packet->get_header( 'call-id' );
+		$self->{response_cache}{$cid} = {
+			packet => $packet,
+			expire => ( $now ||= time()) +32
+		};
 	}
 
 	my $new_entry = Net::SIP::Dispatcher::Packet->new(
@@ -318,45 +285,22 @@ sub deliver {
 
 ###########################################################################
 # cancel delivery of all packets with specific id
-# Args: ($self,$typ?,$id)
-#   $typ: what to cancel, e.g. 'id','callid' or 'qentry', optional,
-#     defaults to 'id' if $id is not ref or 'qentry' if $id is ref
+# Args: ($self,$id)
 #   $id: id to cancel, can also be queue entry
 # Returns: NONE
 ###########################################################################
 sub cancel_delivery {
 	my Net::SIP::Dispatcher $self = shift;
-	my ($callid,$id,$qentry);
-	if ( @_ == 2 ) {
-		my $typ = shift;
-		if ( $typ eq 'callid' ) { $callid = shift }
-		elsif ( $typ eq 'id' ) { $id = shift }
-		elsif ( $typ eq 'qentry' ) { $qentry = shift }
-		else {
-			croak( "bad typ '$typ', should be id|callid|qentry" );
-		}
-	} else {
-		$id = shift;
-		if ( ref($id)) {
-			$qentry = $id;
-			$id = undef;
-		}
-	}
+	my ($id) = @_;
 	my $q = $self->{queue};
-	if ( $qentry ) {
+	if ( ref($id)) {
 		# it's a *::Dispatcher::Packet
-		DEBUG( 100,"cancel packet id: $qentry->{id}" );
-		@$q = grep { $_ != $qentry } @$q;
-	} elsif ( defined $id ) {
-		no warnings; # $_->{id} can be undef
-		DEBUG( 100, "cancel packet id $id" );
-		@$q = grep { $_->{id} ne $id } @$q;
-	} elsif ( defined $callid ) {
-		no warnings; # $_->{callid} can be undef
-		DEBUG( 100, "cancel packet callid $callid" );
-		@$q = grep { $_->{callid} ne $callid } @$q;
+		DEBUG( "cancel packet $id: $id->{id}" );
+		@$q = grep { $_ != $id } @$q;
 	} else {
-		croak( "cancel_delivery w/o id" );
+		no warnings; # $_->{id} can be undef
+		DEBUG( "cancel packet $id" );
+		@$q = grep { $_->{id} ne $id } @$q;
 	}
 }
 
@@ -379,18 +323,13 @@ sub receive {
 	my ($packet,$leg,$from) = @_;
 
 	if ( $packet->is_request ) {
-		my $cache = $self->{response_cache};
-		if ( %$cache ) {
-			my $cid = join( "\0",
-				map { $packet->get_header($_) }
-				qw( cseq call-id from to )
-			);
+		my $cid = $packet->get_header( 'cseq' )
+			."\0".$packet->get_header( 'call-id' );
 
-			if ( my $response = $cache->{$cid} ) {
-				# I have a cached response, use it
-				$self->deliver($response->{packet}, leg => $leg, dst_addr => $from);
-				return;
-			}
+		if ( my $response = $self->{response_cache}{$cid} ) {
+			# I have a cached response, use it
+			$self->deliver($response->{packet}, leg => $leg, dst_addr => $from);
+			return;
 		}
 	}
 
@@ -423,12 +362,12 @@ sub queue_expire {
 
 			if ( !@$retransmits ) {
 				# completly expired
-				DEBUG( 50,"entry %s expired because expire=%.2f but now=%d", $qe->tid,$retransmit,$now );
+				DEBUG( "entry %s expired because expire=%.2f but now=%d", $qe->tid,$retransmit,$now );
 				$changed++;
 				$qe->trigger_callback( ETIMEDOUT );
 
 				# don't put into new queue
-				next;
+				next; 
 			}
 
 			if ( $retransmit ) {
@@ -442,7 +381,7 @@ sub queue_expire {
 			}
 		}
 		push @nq,$qe;
-
+	
 	}
 	$self->{queue} = \@nq if $changed;
 
@@ -479,256 +418,174 @@ sub __deliver {
 	my Net::SIP::Dispatcher $self = shift;
 	my $qentry = shift;
 
-	# loop until leg und dst_addr are known, when we call leg->deliver
-	my $leg = $qentry->{leg}[0];
-	if ( $leg && @{ $qentry->{leg}}>1 ) {
-		DEBUG( 50,"picking first of multiple legs: ".join( " ", map { $_->dump } @{ $qentry->{leg}} ));
-	}
-	my $dst_addr = $qentry->{dst_addr}[0];
+	while (1) {
 
-	if ( ! $dst_addr || ! $leg) {
+		# loop until leg und dst_addr are known, when we call leg->deliver
+		my $leg = $qentry->{leg};
+		my $dst_addr = $qentry->{dst_addr}[0];
 
-		# if explicit routes given use first route
-		# else resolve URI from request
+		if ( ! $dst_addr ) {
 
-		my $uri;
-		my $packet = $qentry->{packet};
-		if ( my ($route) =  $packet->get_header( 'route' )) {
-			($uri) = sip_hdrval2parts( route => $route );
-		} else {
-			$uri = $packet->uri;
-		}
+			DEBUG( "no dst_addr yet" );
 
-		DEBUG( 100,"no dst_addr or leg yet, uri='$uri'" );
-
-		my $callback = sub {
-			my ($self,$qentry,@error) = @_;
-			if ( @error ) {
-				$qentry->trigger_callback(@error);
-				return $self->cancel_delivery( $qentry );
-			} else {
-				$self->__deliver($qentry);
+			if ( my $addr = $self->{outgoing_proxy} ) {
+				# if we have a fixed outgoing proxy use it
+				$qentry->set_dstaddr( $addr );
+				$qentry->{leg} = $self->{outgoing_leg};
+				DEBUG( "setting dst_addr to $addr from outgoing_proxy" );
+				next;
 			}
-		};
-		return $self->resolve_uri(
-			$uri,
-			$qentry->{dst_addr},
-			$qentry->{leg},
-			[ $callback, $self,$qentry ],
-			$qentry->{proto},
-		);
-	}
 
-	# I have leg and addr, send packet thru leg to addr
-	my $cb = sub {
-		my ($self,$qentry,$error) = @_;
-		$self || return;
-		if ( !$error  && $qentry->{retransmits} ) {
-			# remove from queue even if timeout
-			$self->cancel_delivery( $qentry );
+			return $self->__resolve_uri( $qentry, [ \&__deliver,$self,$qentry ] );
+
+		} elsif ( ! $leg ) {
+			# find leg for dst_addr
+			DEBUG( "no leg for dst_addr=$dst_addr yet" );
+			$qentry->{leg} = $self->_find_leg4addr( $dst_addr )
+				|| return $qentry->trigger_callback( EHOSTUNREACH );
+			next;
+
+		} else {
+			# I have leg and addr, send packet thru leg to addr
+			my $cb = sub {
+				my ($qentry,$error) = @_;
+				if ( !$error  && $qentry->{retransmits} ) {
+					# remove from queue even if timeout
+					$self->cancel_delivery( $qentry );
+				}
+				$qentry->trigger_callback( $error );
+			};
+
+			# adds via on cloned packet, calls cb if definite success (tcp)
+			# or error
+			DEBUG( "deliver through leg $leg \@$dst_addr" );
+			$leg->deliver( $qentry->{packet},$dst_addr, [ $cb,$qentry ] );
+
+			if ( !$qentry->{retransmits} ) {
+				# remove from queue if no timeout
+				$self->cancel_delivery( $qentry );
+			}
 		}
-		$qentry->trigger_callback( $error );
-	};
-
-	# adds via on cloned packet, calls cb if definite success (tcp)
-	# or error
-	DEBUG( 50,"deliver through leg ".$leg->dump." \@$dst_addr" );
-	weaken( my $rself = \$self );
-	$cb = [ $cb,$self,$qentry ];
-	weaken( $cb->[1] );
-	$leg->deliver( $qentry->{packet},$dst_addr,$cb );
-
-	if ( !$qentry->{retransmits} ) {
-		# remove from queue if no timeout
-		$self->cancel_delivery( $qentry );
+		last;
 	}
+		
 }
 
 
-
 ###########################################################################
-# resolve URI, determine dst_addr and outgoing leg
-# Args: ($self,$uri,$dst_addr,$legs,$callback;$allowed_proto,$allowed_legs)
-#   $uri: URI to resolve
-#   $dst_addr: reference to list where to put dst_addr
-#   $legs: reference to list where to put leg
-#   $callback: called with () if resolved successfully, else called
-#      with @error
-#   $allowed_proto: optional \@list of protocols (default udp,tcp). If given only
-#      only these protocols will be considered and in this order.
-#   $allowed_legs: optional list of legs which are allowed
+# resolve URI, eg determine dst_addr for URI
+# Args: ($self,$qentry,$callback)
+#   $qentry: *::Dispatcher::Packet
+#   $callback: [ $sub,@arg ], called if resolving is successfully done
+#      if resolving failed it will call the callback from $qentry
 # Returns: NONE
 ###########################################################################
-sub resolve_uri {
+sub __resolve_uri {
 	my Net::SIP::Dispatcher $self = shift;
-	my ($uri,$dst_addr,$legs,$callback,$allowed_proto,$allowed_legs) = @_;
+	my ($qentry,$callback) = @_;
 
+	# we need to get dst_addr from URI
 	# packet should be a request packet (see constructor of *::Dispatcher::Packet)
-	my ($domain,$user,$sip_proto,undef,$param) = sip_uri2parts($uri);
-	$domain or do {
-		DEBUG( 50,"bad URI '$uri'" );
-		return invoke_callback($callback, EHOSTUNREACH );
-	};
+	my $uri = $qentry->{packet}->uri;
+	my $domain = $uri;
+	$domain =~s{^<?sip\w*:}{}i; # strip protocol
+	$domain =~s{.*\@}{};        # strip user part
+	$domain =~s{[>\s;].*}{};    # strip parameters...
+	if ( $domain eq '' ) {
+		# bad URI!
+		$qentry->trigger_callback( EHOSTUNREACH );
+		return $self->cancel_delivery( $qentry );
+	}
 
 	my @proto;
 	my $default_port = 5060;
+	my $sip_proto = 'sip';
 	# XXXX hack, better would be to really parse URI, see *::Util::sip_hdrval2parts
-	if ( $sip_proto eq 'sips' ) {
+	if ( $uri =~m{^sips:}i ) {
 		$default_port = 5061;
 		@proto = 'tcp';
-	} elsif ( my $p = $param->{transport} ) {
+		$sip_proto = 'sips';
+	} elsif ( $uri =~m{;transport="?(udp|tcp)}i ) {
 		# explicit spec of proto
-		@proto = lc($p)
+		@proto = lc($1)
 	} else {
 		# XXXX maybe we should use tcp first if the packet has a specific
 		# minimum length, udp should not be used at all if the packet size is > 2**16
 		@proto = ( 'udp','tcp' );
 	}
 
-	# change @proto so that only the protocols from $allowed_proto are ini it
-	# and that they are tried in the order from $allowed_proto
-	if ( $allowed_proto && @$allowed_proto ) {
-		my @proto_new;
-		foreach my $ap ( @$allowed_proto ) {
-			my $p = first { $ap eq $_ } @proto;
-			push @proto_new,$p if $p;
+	# if domain part of URI looks like 
+	if ( $domain =~m{^\d+\.\d+\.\d+\.\d+(:\d+)?$} ) {
+		# looks like IPv4 address
+		my $addr = $1 ? $domain: $domain.':'.$default_port;
+		$qentry->set_dstaddr( map { "$_:$addr" } @proto );
+		invoke_callback( $callback );
+		return;
+	}
+
+	# do we have a fixed mapping for the domain or upper domain?
+	my $resp;
+	my $d2p = $self->{domain2proxy};
+	if ( $d2p && %$d2p ) {
+		$domain =~s{\.+$}{}; # trailing dots
+		while (1) {
+			$resp = $d2p->{$domain} and last;
+			$domain =~s{^[^\.]+\.}{} or last;
 		}
-		@proto = @proto_new;
-		@proto or do {
-			DEBUG( 50,"no protocols allowed for $uri" );
-			return invoke_callback( $callback, ENOPROTOOPT ); # no proto available
-		};
+		$resp ||= $d2p->{'*'}; # catch-all
 	}
 
-	$dst_addr ||= [];
-	$allowed_legs ||= [ $self->get_legs ];
-	if ( @$legs ) {
-		my %allowed = map { $_ => 1 } @$legs;
-		@$allowed_legs = grep { $allowed{$_} } @$allowed_legs;
-	}
-	@$allowed_legs or do {
-		DEBUG( 50,"no legs allowed for '$uri'" );
-		return invoke_callback($callback, EHOSTUNREACH );
-	};
+	# If no fixed mapping DNS need to be used
+	if ( ! $resp ) {
 
-	my $ip_addr;
-	if ( $domain =~m{^(\d+\.\d+\.\d+\.\d+)(?::(\d+))?$} ) {
-		# if domain part of URI is IPv4[:port]
-		$default_port = $2 if defined $2;
-		$ip_addr = $1;
-		# e.g. 10.0.3.4 should match *.3.0.10.in-addr.arpa
-		$domain = join( '.', reverse split( m{\.},$ip_addr )).'.in-addr.arpa';
-	} else {
-		$domain =~s{\.*(?::(\d+))?$}{}; # remove trailing dots + port
-		$default_port = $1 if defined $1;
-	}
-	DEBUG( 100,"domain=$domain" );
+		# XXXX no full support for RFC3263, eg we don't support NAPTR
+		# but query instead directly for _sip._udp.domain.. like in
+		# RFC2543 specified
 
-	# do we have a fixed proxy for the domain or upper domain?
-	if ( ! @$dst_addr ) {
-		my $d2p = $self->{domain2proxy};
-		if ( $d2p && %$d2p ) {
-			my $dom = $domain;
-			my $addr = $d2p->{$dom}; # exact match
-			while ( ! $addr) {
-				$dom =~s{^[^\.]+\.}{} or last;
-				$addr = $d2p->{ "*.$dom" };
-			}
-			$addr ||= $d2p->{ $dom = '*'}; # catch-all
-			if ( $addr ) {
-				DEBUG( 50,"setting dst_addr from domain specific proxy for domain $dom" );
-				@$dst_addr = @$addr;
+		# XXXX fixme, don't do blocking DNS queries
+		my $dns = Net::DNS::Resolver->new;
+
+		# entries in form [ prio,proto,ip,port ]
+		my @resp;
+
+		# Try to get SRV records for _sip._udp.domain or _sip._tcp.domain
+		foreach my $proto ( @proto ) {
+			if ( my $q = $dns->query( '_'.$sip_proto.'._'.$proto.'.'.$domain,'SRV' )) {
+				foreach my $rr ( $q->answer ) {
+					$rr->type eq 'SRV' || next;
+					# XXX fixme, get IPs for name
+					push @resp,[ $rr->priority, $proto,$rr->name,$rr->port ]
+				}
 			}
 		}
-	}
-
-	# do we have a global outgoing proxy?
-	if ( !@$dst_addr
-		&& ( my $addr = $self->{outgoing_proxy} )) {
-		# if we have a fixed outgoing proxy use it
-		DEBUG( 50,"setting dst_addr+leg to $addr from outgoing_proxy" );
-		@$dst_addr = ( $addr );
-	}
-
-	# is it an IP address?
-	if ( !@$dst_addr && $ip_addr ) {
-		DEBUG( 50,"setting dst_addr from URI because IP address given" );
-		@$dst_addr = ( $ip_addr );
-	}
-
-	# is param maddr set?
-	if ( my $ip = $param->{maddr} ) {
-		@$dst_addr = ( $ip )
-			if $ip =~m{^[\d\.]+$} && eval { inet_aton($ip) };
-	}
-
-	# entries in form [ prio,proto,ip,port ]
-	my @resp;
-	foreach my $addr ( @$dst_addr ) {
-		if ( ref($addr)) {
-			push @resp,$addr; # right format: see domain2proxy
-		} else {
-			$addr =~m{^(?:(udp|tcp):)?([^:]+)(?::(\d+))?$} || next;
-			my $host = $2;
-			my $proto = $1 ? [ $1 ] : \@proto;
-			my $port = $3 ? $3 : $default_port;
-			push @resp, map { [ -1,$_,$host,$port ] } @$proto;
+		# if no SRV records try to resolve address directly
+		unless (@resp) {
+			# try addr directly
+			if ( my $q = $dns->query( $domain,'A' )) {
+				foreach my $rr ($q->answer ) {
+					$rr->type eq 'A' || next;
+					# XXX fixme, get *all* IPs for name
+					push @resp,map {
+						[ -1, $_ , $rr->address,$default_port ]
+					} @proto;
+				}
+			}
 		}
+
+		$resp = \@resp;
+	}
+	DEBUG_DUMP( $resp );
+	if ( ! @$resp ) {
+		$qentry->trigger_callback( EHOSTUNREACH );
+		return $self->cancel_delivery( $qentry );
 	}
 
-	# should we use a fixed transport?
-	if ( my $proto = $param->{transport} ) {
-		@resp = grep { lc($_->[1]) eq lc($proto) } @resp;
-	}
-
-	my @param = ( $dst_addr,$legs,$allowed_legs,$default_port,$callback );
-	return __resolve_uri_final( @param,0,\@resp ) if @resp;
-
-	# If no fixed mapping DNS needs to be used
-
-	# XXXX no full support for RFC3263, eg we don't support NAPTR
-	# but query instead directly for _sip._udp.domain.. like in
-	# RFC2543 specified
-
-	return $self->dns_domain2srv(
-		$domain, \@proto, $sip_proto,
-		[ \&__resolve_uri_final, @param ]
-	);
-}
-
-sub __resolve_uri_final {
-
-	my ($dst_addr,$legs,$allowed_legs,$default_port,$callback,$error,$resp) = @_;
-
-	DEBUG_DUMP( 100,$resp );
-	return invoke_callback( $callback,EHOSTUNREACH )
-		unless $resp && @$resp;
-
-	# for A records we got no port, use default_port
-	$_->[3] ||= $default_port for(@$resp);
 
 	# sort by prio
-	# FIXME: can contradict order in @proto
 	@$resp = sort { $a->[0] <=> $b->[0] } @$resp;
 
-	@$dst_addr = ();
-	@$legs = ();
-	foreach my $r ( @$resp ) {
-		my $leg = first { $_->can_deliver_to(
-			proto => $r->[1],
-			addr  => $r->[2],
-			port  => $r->[3]
-		)} @$allowed_legs;
-
-		if ( $leg ) {
-			push @$dst_addr, "$r->[1]:$r->[2]:$r->[3]";
-			push @$legs,$leg;
-		} else {
-			DEBUG( 50,"no leg for $r->[1]:$r->[2]:$r->[3]" );
-		}
-	}
-
-	return invoke_callback( $callback, EHOSTUNREACH ) if !@$dst_addr;
+	my @addr = map { "$_->[1]:$_->[2]:$_->[3]" } @$resp;
+	$qentry->set_dstaddr( @addr );
 	invoke_callback( $callback );
 }
 
@@ -736,115 +593,12 @@ sub __resolve_uri_final {
 sub _find_leg4addr {
 	my Net::SIP::Dispatcher $self = shift;
 	my $dst_addr = shift;
-	my ($proto,$ip) = $dst_addr =~m{^(?:(tcp|udp):)?([^:]+)};
-	my @legs;
-	foreach my $leg (@{ $self->{legs} }) {
-		push @legs,$leg if $leg->can_deliver_to( addr => $ip, proto => $proto );
+	my $legs = $self->{legs};
+	return $legs->[0] if @$legs == 1;
+	foreach my $leg (@$legs) {
+		return $leg if $leg->can_deliver_to( $dst_addr );
 	}
-	return @legs;
-}
-
-###########################################################################
-# resolve hostname to IP using DNS
-# FIXME: should work asynchronously
-# Args: ($self,$host,$callback)
-#   $host: hostname or hash with hostname as keys
-#   $callback: gets called with (EINVAL) or (undef,result) once finished
-#     result is IP for single hosts or the input hash ref where the
-#     IPs are filled in as values
-# Returns: NONE
-###########################################################################
-sub dns_host2ip {
-	my Net::SIP::Dispatcher $self = shift;
-	my ($host,$callback) = @_;
-	if ( ref($host)) {
-		my $err;
-		foreach ( keys %$host ) {
-			if ( my $addr = gethostbyname( $_ )) {
-				$host->{$_} = inet_ntoa($addr);
-			} else {
-				$err = EINVAL;
-			}
-		}
-		invoke_callback( $callback, $err,$host );
-	} else {
-		my $addr = gethostbyname( $host );
-		invoke_callback( $callback, $addr ? ( undef,inet_ntoa($addr) ) : ( $? ));
-	}
-}
-
-###########################################################################
-# get SRV records using DNS
-# FIXME: should work asynchronously
-# Args: ($self,$domain,$proto,$sip_proto,$callback)
-#   $domain: domain for SRV query
-#   $proto: which protocols to check
-#   $sip_proto: sip|sips
-#   $callback: gets called with result once finished
-#      result is \@list of [ prio,proto,name,port ]
-# Returns: NONE
-###########################################################################
-sub dns_domain2srv {
-	my Net::SIP::Dispatcher $self = shift;
-	my ($domain,$protos,$sip_proto,$callback) = @_;
-
-	# FIXME: don't do blocking DNS queries
-	my $dns = Net::DNS::Resolver->new;
-
-	# Try to get SRV records for _sip._udp.domain or _sip._tcp.domain
-	my (@resp,%addr2ip);
-	foreach my $proto ( @$protos ) {
-		if ( my $q = $dns->query( '_'.$sip_proto.'._'.$proto.'.'.$domain,'SRV' )) {
-			foreach my $rr ( $q->answer ) {
-				if ( $rr->type eq 'A' ) {
-					push @{ $addr2ip{$rr->name} }, $rr->address;
-				} elsif ( $rr->type eq 'SRV' ) {
-					push @resp,[ $rr->priority, $proto,$rr->target,$rr->port ]
-				}
-			}
-		}
-	}
-
-	# name to addr based on additional records in DNS answer
-	my @resp_resolved;
-	for my $r (@resp) {
-		if ( my $addr = $addr2ip{ $r->[2] } ) {
-			for (@$addr) {
-				my @cp = @$r;
-				$cp[2] = $_;
-				push @resp_resolved, \@cp;
-			}
-		} else {
-			# either already IP or no additional data for resolving -> later
-			my @cp = @$r;
-			# XXX fixme blocking DNS lookup
-			my $ipn = gethostbyname( $r->[2] ) or do {
-				DEBUG( 1,"cannot resolve $r->[2]" );
-				next;
-			};
-			$cp[2] = inet_ntoa($ipn);
-			push @resp_resolved, \@cp;
-		}
-	}
-	@resp = @resp_resolved;
-
-	# if no SRV records try to resolve address directly
-	unless (@resp) {
-		# try addr directly
-		my $default_port = $sip_proto eq 'sips' ? 5061:5060;
-		if ( my $q = $dns->query( $domain,'A' )) {
-			foreach my $rr ($q->answer ) {
-				$rr->type eq 'A' || next;
-				# XXX fixme, check that name in response corresponds to query
-				# (beware of CNAMEs!)
-				push @resp,map {
-					[ -1, $_ , $rr->address,$default_port ]
-				} @$protos;
-			}
-		}
-	}
-	my $error = @resp ? 0 : EINVAL;
-	invoke_callback( $callback,$error,\@resp );
+	return; # nothing found
 }
 
 ###########################################################################
@@ -852,20 +606,17 @@ sub dns_domain2srv {
 # Container for Queue entries in Net::SIP::Dispatchers queue
 ###########################################################################
 package Net::SIP::Dispatcher::Packet;
-use fields (
+use fields ( 
 	'id',           # transaction id, used for canceling delivery if response came in
-	'callid',       # callid, used for canceling all deliveries for this call
 	'packet',       # the packet which nees to be delivered
+	'leg',          # through which leg the packet gets delivered
 	'dst_addr',     # to which adress the packet gets delivered, is array-ref because
-					# the DNS/SRV lookup might return multiple addresses and protocols
-	'leg',          # through which leg the packet gets delivered, same number
-					# of items like dst_addr
+	                # the DNS/SRV lookup might return multiple addresses and protocols
 	'retransmits',  # array of retransmit time stamps, if undef no retransmit will be
-					# done, if [] no more retransmits can be done (trigger ETIMEDOUT)
+	                # done, if [] no more retransmits can be done (trigger ETIMEDOUT)
 					# the last element in this array will not used for retransmit, but
 					# is the timestamp, when the delivery fails permanently
 	'callback',     # callback for DSN (success, ETIMEDOUT...)
-	'proto',        # list of possible protocols, default tcp and udp for sip:
 );
 
 use Net::SIP::Debug;
@@ -885,17 +636,10 @@ sub new {
 	my $self = fields::new( $class );
 	%$self = %args;
 	$self->{id} ||= $self->{packet}->tid;
-	$self->{callid} ||= $self->{packet}->callid;
 
 	if ( my $addr = $self->{dst_addr} ) {
 		$self->{dst_addr} = [ $addr ] if !ref($addr)
 	}
-	if ( my $leg = $self->{leg} ) {
-		$self->{leg} = [ $leg ] if UNIVERSAL::can( $leg,'deliver' );
-	}
-
-	$self->{dst_addr} ||= [];
-	$self->{leg} ||= [];
 
 	# figure out retransmit times
 	my $p = $self->{packet} || die "no packet for delivery";
@@ -956,12 +700,25 @@ sub prepare_retransmits {
 		$to = $t2 if $t2 && $to>$t2;
 		$rtm += $to
 	}
-	DEBUG( 100,"retransmits $now + ".join( " ", map { $_ - $now } @retransmits ));
+	DEBUG( "retransmits $now + ".join( " ", map { $_ - $now } @retransmits ));
 	$self->{retransmits} = \@retransmits;
 }
 
 
 
+###########################################################################
+# set dst_addr
+# Args: ($self,@dst_addr)
+#   @dst_addr: hash with ip:port (ip, not hostname !!) or
+#     proto:ip:port, where proto is udp|tcp. If no proto is
+#     given udp is assumed
+# Returns: NONE
+###########################################################################
+sub set_dstaddr {
+	my Net::SIP::Dispatcher::Packet $self = shift;
+	$self->{dst_addr} = [ @_ ]
+}
+
 ###########################################################################
 # use next dst_addr (eg if previous failed)
 # Args: $self
@@ -972,8 +729,6 @@ sub use_next_dstaddr {
 	my Net::SIP::Dispatcher::Packet $self = shift;
 	my $addr = $self->{dst_addr} || return;
 	shift(@$addr);
-	my $leg = $self->{leg} || return;
-	shift(@$leg);
 	return @$addr && $addr->[0];
 }
 
@@ -989,7 +744,7 @@ sub trigger_callback {
 	my Net::SIP::Dispatcher::Packet $self = shift;
 	my $error = shift;
 	my $cb = $self->{callback} || return;
-	invoke_callback( $cb,$error,$self);
+	invoke_callback( $cb, $error ? ($error,$self):() );
 	return 1;
 }
 
