@@ -14,7 +14,6 @@ use Socket;
 use List::Util qw(first);
 use Net::SIP::Util 'invoke_callback';
 use Net::SIP::Debug;
-use Errno 'EINTR';
 
 ###########################################################################
 # creates new event loop
@@ -34,19 +33,18 @@ sub new {
 
 ###########################################################################
 # adds callback for the event, that FD is readable
-# Args: ($self,$fd,$callback,?$name)
+# Args: ($self,$fd,$callback)
 #  $fd: file descriptor
 #  $callback: callback to be called, when fd is readable, will be called
 #    with fd as argument
-#  $name: optional name for callback, used for debugging
 # Returns: NONE
 ###########################################################################
 sub addFD {
 	my Net::SIP::Dispatcher::Eventloop $self = shift;
-	my ($fd,$callback,$name) = @_;
+	my ($fd,$callback) = @_;
 	defined( my $fn = fileno($fd)) || return;
-	#DEBUG( 100, "$self added fn=$fn sock=".eval { my ($port,$addr) = unpack_sockaddr_in( getsockname($fd)); inet_ntoa($addr).':'.$port } );
-	$self->{fd}[$fn] = [ $fd,$callback,$name ];
+	#DEBUG( "$self added fn=$fn sock=".eval { my ($port,$addr) = unpack_sockaddr_in( getsockname($fd)); inet_ntoa($addr).':'.$port } );
+	$self->{fd}[$fn] = [ $fd,$callback ];
 }
 
 ###########################################################################
@@ -59,27 +57,24 @@ sub delFD {
 	my Net::SIP::Dispatcher::Eventloop $self = shift;
 	my ($fd) = @_;
 	defined( my $fn = fileno($fd)) || return;
-	#DEBUG( 100, "$self delete fn=$fn sock=".eval { my ($port,$addr) = unpack_sockaddr_in( getsockname($fd)); inet_ntoa($addr).':'.$port } );
+	#DEBUG( "$self delete fn=$fn sock=".eval { my ($port,$addr) = unpack_sockaddr_in( getsockname($fd)); inet_ntoa($addr).':'.$port } );
 	delete $self->{fd}[$fn];
 }
 
 ###########################################################################
 # add timer
-# Args: ($self,$when,$callback;$repeat,$name)
+# Args: ($self,$when,$callback;$repeat)
 #  $when: absolute time_t or relative (smaller than a year), can be
 #    subsecond resolution
 #  $callback: callback to be called, gets timer object as argument
 #  $repeat: interval for repeated callbacks, optional
-#  $name: optional name for debugging
-# Returns: $timer object
 ###########################################################################
-sub add_timer {
+sub addTimer {
 	my Net::SIP::Dispatcher::Eventloop $self = shift;
-	my ($when,$callback,$repeat,$name ) = @_;
+	my ($when,$callback,$repeat ) = @_;
 	$when += $self->{now} if $when < 3600*24*365;
-
-	my $timer = Net::SIP::Dispatcher::Eventloop::TimerEvent->new(
-		$when, $repeat, $callback,$name );
+		
+	my $timer = Net::SIP::Dispatcher::Eventloop::TimerEvent->new( $when, $repeat, $callback );
 	push @{ $self->{timer}}, $timer;
 	return $timer;
 }
@@ -117,7 +112,7 @@ sub loop {
 
 	while ( !$to || $to>0 ) {
 
-		DEBUG( 100, "timeout = ".( defined($to) ? $to: '<undef>' ));
+		DEBUG( "timeout = ".( defined($to) ? $to: '<undef>' ));
 		# handle timers
 		my $timer = $self->{timer};
 
@@ -132,15 +127,14 @@ sub loop {
 			# run expired timers
 			while ( @$timer && $timer->[0]{expire} <= $looptime ) {
 				my $t = shift(@$timer);
-				DEBUG( 50, "trigger timer(%s) %s repeat=%s",
-					$t->name,$t->{expire} || '<undef>', $t->{repeat} || '<undef>' );
-				invoke_callback( $t->{callback},$t );
-				if ( $t->{expire} && $t->{repeat} ) {
+				DEBUG( "trigger timer %s repeat=%s",$t->{expire} || '<undef>', $t->{repeat} || '<undef>' );
+				if ( $t->{repeat} ) {
 					$t->{expire} += $t->{repeat};
-					DEBUG( 100, "timer(%s) gets repeated at %d",$t->name,$t->{expire} );
+					DEBUG( "timer gets repeated at $t->{expire}" );
 					push @$timer,$t;
 					$do_timer = 1; # rerun loop
 				}
+				invoke_callback( $t->{callback},$t );
 			}
 		}
 
@@ -149,13 +143,13 @@ sub loop {
 			my $next_timer = $timer->[0]{expire} - $looptime;
 			$to = $next_timer if !defined($to) || $to>$next_timer;
 		}
-		DEBUG( 100, "timeout = ".( defined($to) ? $to: '<undef>' ));
+		DEBUG( "timeout = ".( defined($to) ? $to: '<undef>' ));
 
 		if ( grep { ${$_} } @stop ) {
-			DEBUG( 50, "stopvar triggered" );
+			DEBUG( "stopvar triggered" );
 			return;
 		}
-
+		
 		# wait for selected fds
 		my $fds = $self->{fd};
 		my $rin;
@@ -164,22 +158,19 @@ sub loop {
 			# Select which fds are readable or timeout
 			my $rin = '';
 			map { vec( $rin,fileno($_->[0]),1 ) = 1 } @to_read;
-			DEBUG( 100, "handles=".join( " ",map { fileno($_->[0]) } @to_read ));
-			select( my $rout = $rin,undef,undef,$to ) < 0 and do {
-				next if $! == EINTR;
-				die $!
-			};
+			DEBUG( "$self handles=".join( " ",map { fileno($_->[0]) } @to_read ));
+			die $! if select( my $rout = $rin,undef,undef,$to ) < 0;
+			my @can_read = grep { vec($rout,fileno($_->[0]),1) } @to_read;
+			DEBUG( "$self can_read=".join( " ",map { fileno($_->[0]) } @can_read ));
+
 			# returned from select
 			$looptime = $self->{now} = gettimeofday();
-			DEBUG( 100, "can_read=".join( " ",map { $_ } grep { $fds->[$_] && vec($rout,$_,1) } (0..$#$fds)));
-			for( my $fn=0;$fn<@$fds;$fn++ ) {
-				vec($rout,$fn,1) or next;
-				my $fd_data = $fds->[$fn] or next;
-				DEBUG( 50,"call cb on fn=$fn ".( $fd_data->[2] || '') );
+
+			foreach my $fd_data (@can_read) {
 				invoke_callback( $fd_data->[1],$fd_data->[0] );
 			}
 		} else {
-			DEBUG( 50, "no handles, sleeping for %s", defined($to) ? $to : '<endless>' );
+			DEBUG( "no handles, sleeping for %s", defined($to) ? $to : '<endless>' );
 			select(undef,undef,undef,$to )
 		}
 
@@ -194,37 +185,21 @@ sub loop {
 
 
 ##########################################################################
-# Timer object which gets returned from add_timer and has method for
+# Timer object which gets returned from addTimer and has method for
 # canceling the timer (by setting expire to 0)
 ##########################################################################
 package Net::SIP::Dispatcher::Eventloop::TimerEvent;
-use fields qw( expire repeat callback name );
+use fields qw( expire repeat callback );
 
 ##########################################################################
-# create new timer object, see add_timer for description of Args
+# create new timer object, see addTimer for description of Args
 # Args: ($class,$expire,$repeat,$callback)
 # Returns: $self
 ##########################################################################
 sub new {
-	my ($class,$expire,$repeat,$callback,$name) = @_;
+	my ($class,$expire,$repeat,$callback) = @_;
 	my $self = fields::new( $class );
-	unless ( $name ) {
-		# check with caller until I find a function which is not
-		# named 'add_timer'
-		for( my $i=1;1;$i++ ) {
-			my (undef,undef,undef,$sub) = caller($i) or last;
-			next if $sub =~m{::add_timer$};
-			my $line = (caller($i-1))[2];
-			$name = "${sub}[$line]";
-			last;
-		}
-	}
-	%$self = (
-		expire => $expire,
-		repeat => $repeat,
-		callback => $callback,
-		name => $name
-	);
+	%$self = ( expire => $expire, repeat => $repeat, callback => $callback );
 	return $self;
 }
 
@@ -237,17 +212,6 @@ sub new {
 sub cancel {
 	my Net::SIP::Dispatcher::Eventloop::TimerEvent $self = shift;
 	$self->{expire} = 0;
-	$self->{callback} = undef;
-}
-
-##########################################################################
-# returns name for debugging
-# Args: $self
-# Returns: $name
-##########################################################################
-sub name {
-	my Net::SIP::Dispatcher::Eventloop::TimerEvent $self = shift;
-	return $self->{name} || 'NONAME'
 }
 
 1;
