@@ -176,6 +176,18 @@ sub loop {
 	return $self->{loop}->loop( $timeout,@stopvar );
 }
 
+###########################################################################
+# add timer
+# propagates to add_timer of wNet::SIP::Dispatcher, see there for detailed 
+# explanation of args
+# Args: ($self,$when,$cb,$repeat) 
+# Returns: $timer
+###########################################################################
+sub add_timer {
+	my Net::SIP::Simple $self = shift;
+	$self->{dispatcher}->add_timer( @_ );
+}
+
 ###########################################################################
 # control RTP behavior
 # Args: ($self,$method,@arg)
