@@ -12,23 +12,17 @@ package Net::SIP::Util;
 use Digest::MD5 'md5_hex';
 use IO::Socket;
 use Net::SIP::Debug;
-use Carp qw(confess croak);
 use base 'Exporter';
 
-our @EXPORT_OK = qw(
-	sip_hdrval2parts
+our @EXPORT_OK = qw( 
+	sip_hdrval2parts 
 	sip_parts2hdrval
-	sip_uri2parts
-	create_socket_to
-	create_rtp_sockets
-	invoke_callback
-	sip_uri_eq
+	create_socket_to 
+	create_rtp_sockets 
+	invoke_callback 
 );
 our %EXPORT_TAGS = ( all => \@EXPORT_OK );
 
-our $RTP_MIN_PORT = 2000;
-our $RTP_MAX_PORT = 12000;
-
 ###########################################################################
 # creates hash from header val, e.g.
 # 'Digest method="md5",qop="auth",...','www-authenticate' will result in
@@ -41,7 +35,6 @@ our $RTP_MAX_PORT = 12000;
 #   %parameter: additional parameter
 ###########################################################################
 sub sip_hdrval2parts {
-	croak( "usage: sip_hdrval2parts( key => val )" ) if @_!=2;
 	my ($key,$v) = @_;
 	return if !defined($v);
 	my $delim = ';';
@@ -54,30 +47,19 @@ sub sip_hdrval2parts {
 	# split on delimiter (but not if quoted)
 	my @v = ('');
 	my $quoted = 0;
-	my $bracket = 0;
 	while (1) {
-		if ( $v =~m{\G(.*?)([\\"<>$delim])}gc ) {
+		if ( $v =~m{\G(.*?)([\\"$delim])}gc ) {
 			if ( $2 eq "\\" ) {
 				$v[-1].=$1.$2.substr( $v,pos($v),1 );
 				pos($v)++;
 			} elsif ( $2 eq '"' ) {
 				$v[-1].=$1.$2;
-				$quoted = !$quoted if ! $bracket;
-			} elsif ( $2 eq '<' ) {
-				$v[-1].=$1.$2;
-				$bracket = 1 if ! $bracket && ! $quoted;
-			} elsif ( $2 eq '>' ) {
-				$v[-1].=$1.$2;
-				$bracket = 0 if $bracket && ! $quoted;
+				$quoted = !$quoted;
 			} elsif ( $2 eq $delim ) {
 				# next item if not quoted
-				if ( ! $quoted && ! $bracket ) {
-					( $v[-1].=$1 ) =~s{\s+$}{}; # strip trailing space
-					push @v,'' ;
-					$v =~m{\G\s+}gc; # skip space after $delim
-				} else {
-					$v[-1].=$1.$2
-				}
+				( $v[-1].=$1 ) =~s{\s+$}{}; # strip trailing space
+				push @v,'' if !$quoted;
+				$v =~m{\G\s+}gc; # skip space after $delim
 			}
 		} else {
 			# add rest to last from @v
@@ -141,52 +123,7 @@ sub sip_parts2hdrval {
 	}
 	return $val;
 }
-
-
-###########################################################################
-# extract parts from SIP URI
-# Args: $uri
-# Returns: $domain || ($domain,$user,$proto,$data,$param)
-#  $domain: SIP domain maybe with port
-#  $user:   user part
-#  $proto:  'sip'|'sips'
-#  $data:   full part before any params
-#  $param:  hashref with params, e.g { transport => 'udp',... }
-###########################################################################
-sub sip_uri2parts {
-	my $uri = shift;
-	$uri = $1 if $uri =~m{<([^>]+)>\s*$}i;
-	my ($data,$param) = sip_hdrval2parts( uri => $uri );
-	if ( $data =~m{^(?:(sips?):)?(?:([^\s\@]*)\@)?([\w\-\.:]+)}i ) {
-		my ($proto,$user,$domain) = ($1,$2,$3);
-		$proto ||= 'sip';
-		return wantarray
-			? ($domain,$user,lc($proto),$data,$param)
-			: $domain
-	} else {
-		return;
-	}
-}
-
-###########################################################################
-# returns true if two URIs are the same
-# Args: $uri1,$uri2
-# Returns: true if both URI point to same address
-###########################################################################
-sub sip_uri_eq {
-	my ($uri1,$uri2) = @_;
-	return 1 if $uri1 eq $uri2; # shortcut for common case
-	my ($d1,$u1,$p1) = sip_uri2parts($uri1);
-	my ($d2,$u2,$p2) = sip_uri2parts($uri2);
-	my $port1 = $d1 =~s{:(\d+)$|\[(\d+)\]$}{} ? $1||$2
-		: $p1 eq 'sips' ? 5061 : 5060;
-	my $port2 = $d2 =~s{:(\d+)$|\[(\d+)\]$}{} ? $1||$2
-		: $p2 eq 'sips' ? 5061 : 5060;
-	return lc($d1) eq lc($d2)
-		&& $port1 == $port2
-		&& ( defined($u1) ? defined($u2) && $u1 eq $u2 : ! defined($u2))
-		&& $p1 eq $p2;
-}
+	
 
 ###########################################################################
 # create socket preferable on port 5060 from which one might reach the given IP
@@ -197,7 +134,7 @@ sub sip_uri_eq {
 #  $sock: the created socket
 #  $ip_port: ip:port of socket, only given if called in array context
 # Comment: the IP it needs to come from works by creating a udp socket
-#  to this host and figuring out it's IP by calling getsockname. Then it
+#  to this host and figuring out it's IP by calling getsockname. Then it 
 #  tries to create a socket on this IP using port 5060 and if this does
 #  not work it tries the port 5062..5100 and if this does not work too
 #  it let the system use a random port
@@ -209,10 +146,10 @@ sub create_socket_to {
 
 	my $laddr = do {
 		$dst_addr =~s{:.*}{}; # in case ip:port was given
-		my $sock = IO::Socket::INET->new(
-			PeerAddr => $dst_addr,
+		my $sock = IO::Socket::INET->new( 
+			PeerAddr => $dst_addr, 
 			PeerPort => 5060,
-			Proto => 'udp'
+			Proto => 'udp' 
 		) || return; # No route?
 		my $x = getsockname($sock) or return;
 		my (undef,$addr) = unpack_sockaddr_in( $x );
@@ -226,7 +163,7 @@ sub create_socket_to {
 	my ($sock,$port);
 	for my $p ( 5060,5062..5100 ) {
 		DEBUG( "try to listen on $laddr:$p" );
-		$sock = IO::Socket::INET->new(
+		$sock = IO::Socket::INET->new( 
 			LocalAddr => $laddr,
 			LocalPort => $p,
 			Proto => $proto,
@@ -237,7 +174,7 @@ sub create_socket_to {
 		}
 	}
 	if ( ! $sock ) {
-		$sock = IO::Socket::INET->new(
+		$sock = IO::Socket::INET->new( 
 			LocalAddr => $laddr, # use any port
 			Proto => $proto,
 		) || return;
@@ -253,9 +190,8 @@ sub create_socket_to {
 # Args: ($laddr;$range,$min,$max,$tries)
 #   $laddr: local addr
 #   $range: how many sockets, 2 if not defined
-#   $min: minimal port number, default $RTP_MIN_PORT
+#   $min: minimal port number, default 2000
 #   $max: maximal port number, default 10000 more than $min
-#      or $RTP_MAX_PORT if $min not given
 #   $tries: how many tries, default 100
 # Returns: ($port,$rtp_sock,$rtcp_sock,@more_socks)
 #   $port:      port of RTP socket, port for RTCP is port+1
@@ -266,13 +202,9 @@ sub create_socket_to {
 sub create_rtp_sockets {
 	my ($laddr,$range,$min,$max,$tries) = @_;
 	$range ||= 2;
-	if ( ! $min ) {
-		$min = $RTP_MIN_PORT;
-		$max ||= $RTP_MAX_PORT;
-	} else {
-		$max ||= $min+10000;
-	}
+	$min ||= 2000;
 	$min += $min%2; # make even
+	$max ||= $min+10000;
 	$tries ||= 1000;
 
 	my $diff2 = int(($max-$min)/2) - $range +1;
@@ -298,7 +230,7 @@ sub create_rtp_sockets {
 }
 
 ###########################################################################
-# helper to call callback, set variable..
+# helper to call callback, set variable.. 
 # Args: ($cb;@args)
 #  $cb:  callback
 #  @args: additional args for callback
@@ -317,6 +249,10 @@ sub invoke_callback {
 	if ( UNIVERSAL::isa( $cb,'CODE' )) {
 		# anon sub
 		return $cb->(@more_args)
+	} elsif ( UNIVERSAL::isa( $cb,'SCALAR' )) {
+		# scalar ref, set to true
+		$$cb = @more_args ? shift(@more_args) : 1;
+		return $$cb;
 	} elsif ( my $sub = UNIVERSAL::can( $cb,'run' )) {
 		# Callback object
 		return $sub->( @more_args );
@@ -330,12 +266,8 @@ sub invoke_callback {
 			return 1 if m{$cb}
 		}
 		return 0;
-	} elsif ( UNIVERSAL::isa( $cb,'SCALAR' ) || UNIVERSAL::isa( $cb,'REF' )) {
-		# scalar ref, set to true
-		$$cb = @more_args ? shift(@more_args) : 1;
-		return $$cb;
 	} elsif ( $cb ) {
-		confess "unknown handler $cb";
+		die "unknown handler $cb"
 	}
 }
 
